# snap-ml-spark in PowerAI Enterprise

Applications using `snap-ml-spark` APIs can be executed in Spectrum Conductor
environment in IBM PowerAI Enterprise using the set of files provided in this directory.

The following operations are supported,

(1) run `snap-ml-spark` applications through `spark-submit` in PowerAI Enterprise

(2) enable `snap-ml-spark` APIs inside Jupyter Notebooks in PowerAI Enterprise

To perform these operations, a set of configuration changes in IBM Spectrum Conductor is required.

## Configuring IBM Spectrum Conductor with Spark for IBM PowerAI SnapML

### Create gpus Resource Group

Log in to the `IBM Spectrum Computing Cluster Management Console` as an Administrator.

From the cluster management console, navigate to Resources > Resource Planning (Slot) > Resource Groups. Under Global Actions, click Create a Resource Group. Create a resource group called `gpus` with Advanced Formula `ngpus`.

Navigate to Resources > Consumers and select a consumer, such as `SampleApplications`. Select it and click `Consumer Properties` tab. Under the section `Specify slot-based resource groups`, Select ithe check mark for the resource group that was just created (`gpus`) and click Apply.

Navigate to Resources > Resource Planning (Slot) > Resource Plan. Select `Resource Group: gpus` from the drop down box and `Exclusive` radio button as the Slot allocation policy, to indicate that when IBM Spectrum Conductor with Spark allocates resources from this resource group, using all free slots from a host. For example, assuming there are 4 GPUs on a host, a request for 1, 2, 3, or 4 GPUs would take the whole host. Click `Apply`.

### Create Anaconda environment

Create an anaconda environment first and then during creation of Spark Instance Group(SIG), click on the checkmark for `Jupyter Notebook` and select the anaconda distribution and the anaconda environment name from the drop down boxes.

Navigate to `Workload -> Spark -> Anaconda Management`. Click on `Deploy` after selecting ppc64le anaconda distribution name like `Anaconda5-1-0-Python3-Linux-ppc64le`. Give a name (say `myAnaconda`) for the anaconda distribution and a deployment directory (say `/home/egoadmin/myAnaconda`). Click `Deploy`.

Click on ppc64le anaconda distribution name like `Anaconda5-1-0-Python3-Linux-ppc64le` and in the wizard, click on the Anaconda distribution instance `myAnaconda` -> Click `Add` under `Conda environments` -> Deselect the check mark `Create environment from a yaml file` and provide an environment name (say `env1`). Click `Add`.

### Create Spark Instance Group

To use snap-ml-spark, Spark Instance Group(SIG) is to be configured in Spectrum Conductor with specific configurations as given below.

Navigate to `Workload -> Spark -> Spark Instance Groups -> New`.
Select `Spark 2.3.1` version from the drop down box. The `Configuration` link
near `Spark 2.3.1` is to be used to set the configuration properties
`SPARK_EGO_CONF_DIR_EXTRA`, `SPARK_EGO_GPU_EXECUTOR_SLOTS_MAX` and `spark.jars`.

In the Spark 2.3.1 Configuration wizard,

(1) The value to be set for the property `SPARK_EGO_CONF_DIR_EXTRA` is `$CONDA_PREFIX/snap-ml-spark/conductor_spark/conf`.

(2) The value to be set for the property `SPARK_EGO_GPU_EXECUTOR_SLOTS_MAX` is the number of GPUs available on each host in the cluster. For example, `SPARK_EGO_GPU_EXECUTOR_SLOTS_MAX=4`.

(3) Go to `Additional Parameters` -> `Add a Parameter` button -> add the parameter `spark.jars` with the value `$CONDA_PREFIX/snap-ml-spark/lib/snap-ml-spark-v*-ppc64le.jar`.


Select the check mark `Jupyter 5.4.0` at `Enable notebooks`.

Provide a shared directory as the base data directory (say `/paie-nfs/data`).

Select the `Anaconda distribution instance` and the `Conda environment` from the drop down boxes.

In the `Configuration` link for `Jupyter 5.4.0`, go to `Environment Variables` tab and click `Add a variable` and add the variable `JUPYTER_SPARK_OPTS` with the value `--conf spark.ego.gpu.app=true --conf spark.ego.gpu.executors.slots.max=4 --conf spark.default.parallelism=8` to use 8 GPUs with 8 partitions for notebooks where 2 hosts with 4GPUs on each host exists in the SIG.

Under `Resource Groups and Plans` section, `gpus` resource group is selected for `Spark Executors (GPU slots)`. `ComputeHosts` resource group is selected for all other things under `Resource Groups and Plans` section.

Once the SIG is deployed and started, Click on the SIG, go to `Notebooks` tab, click on `Create Notebooks for Users`. Select users (say `Admin` and other users, as required) and click `Create`.

Stop and start the created `Jupyter 5.4.0` notebook once to get the sample notebooks to home page containing `snap_ml_spark_example_notebooks` folder when you login to the Jupyter Notebook home page after performing `Create Notebook for Users` in `Notebooks` tab in SIG.

## How to run snap-ml-spark applications through spark-submit in PowerAI Enterprise

In the Cluster Management Console(GUI), navigate to

Workload -> Spark -> My Applications And Notebooks -> `Run Application` button for spark-submit.

Sample spark-submit command takes the following arguments in the box in the `Run Application` wizard:

```
--master ego-client --conf spark.ego.gpu.app=true $CONDA_PREFIX/snap-ml-spark/examples/example-criteo45m/example-criteo45m.py --data_path /tmp/criteoData --num_partitions 8 --use_gpu
```

In the above spark-submit command arguments, `ego-client` is to be replaced with `ego-cluster` to submit Spark job in cluster mode instead of client mode.

Here in this spark-submit command, we have used one of the examples `example-criteo45m.py` which is shipped with PowerAI base package. `/tmp/criteoData/data/` directory should contain the input criteo data. This data directory is a directory on the host in the cluster where the selected spark master (selected in `Run Application` page) is running. Details related to how to run this example and its related dataset can be found in `$CONDA_PREFIX/snap-ml-spark/examples/example-criteo45m/README.md`. Few more examples are available in `$CONDA_PREFIX/snap-ml-spark/examples`.

## How to run Jupyter Notebooks in PowerAI Enterprise using snap-ml-spark

Click on the SIG, go to `Notebooks` tab.

Start the Notebook if not in started satte.

Click on `My Notebooks` drop down box and click on something like `Jupyter 5.4.0 - owned by Admin`. This would open a new window with login to Home page of Notebooks.

Login as Admin user and click on `snap_ml_spark_example_notebooks` folder and click on any of the sample notebooks to open and run.

&copy; Copyright IBM Corporation 2018, 2019
